#!/bin/bash

# Script saved from Dejal Simon 2.5 <http://www.dejal.com/simon/>
# 
# Saved by David Sinclair on 2009-02-18 12:13:36
# 
# Script is for Bourne-Again Shell: /bin/bash

# *** INSTRUCTIONS: ***
# 
# Script to check drive capacity threshold, S.M.A.R.T. status, and RAID 
# health by Scott Haneda scott@hostizard.com 
# 
# Any single failure is a failure to be looked into.  This test is not 
# to alert you which of RAID, Capacity, and S.M.A.R.T. failed, but 
# rather, to alwert you to look at your drives and see what is wrong. 
# 
# Set Debug to 1 if you are having troubles Set Max Capacity to a 
# percentage value that your drive should not exceed (ie: 90). 

# *** CUSTOM VARIABLES: ***
# 
# Debug: 1
# Max Capacity: 90

# *** CUSTOM RESULTS: ***
# 
# 0: Success
# 1: RAID, CAPACITY or SMART failure

# *** SCRIPT: ***

#Debug this file?
debug={Debug} # Set to 0 to turn off all debugging
if [[ $debug = 1 ]]; then echo "(INFO) DEBUGGING is ON"; fi 

# Where do you want this output saved?
# Needs to be somewhere that http can serve it on this machine
# Does not end in trailing slash
save_path=/tmp

# Filename you want to save the results to
save_filename=drive_status.txt

# Threshold for when a drive is too full, in integer, as percent
drive_capacity_thresh={Max Capacity}

# -- DO NOT EDIT BELOW THIS LINE -- #

# Just in case, make the path, so we can avoid error checking it
mkdir -p $save_path

# Set the path to the entire filename
save_path=$save_path/$save_filename

if [[ $debug = 1 ]]; then echo "(INFO) save_path: $save_path"; fi;


# --
# Start check capacity
# --

# Clear out the $save_path file, add a date into the file
echo '' > $save_path
echo "File last updated on: `date`" >> $save_path

# Check the `df` or capacity status on all drives
df | grep '/dev/' | while read df_line; 
do
	# Get the volume identifier
	volume_id=`echo $df_line | awk '{print $1;}'`
	
	# Must ignore disk images, they already exist on the main mount, so why bother checking
	# grep EXIT STATUS 0 means found, 1 means no match
	is_dmg=`diskutil info $volume_id | grep 'Disk Image'`
	if [[ $? -eq 0 ]]; then
		if [[ $debug = 1 ]]; then echo "(Capacity) Found a .dmg, aborting capacity check"; fi;
		continue
	fi
		
	# Get the current used space of the volume
	volume_used=`echo $df_line | awk '{print $5;}' | sed 's/%//g'`

	if [[ $volume_used -ge $drive_capacity_thresh ]]
	then
		if [[ $debug = 1 ]]; then echo "(Capacity) $volume_id is more than $drive_capacity_thresh% full"; fi;
		echo "Capacity ($volume_id $volume_used%): FAILURE" >> $save_path
	else
		if [[ $debug = 1 ]]; then echo "(Capacity) $volume_id is less than $drive_capacity_thresh% full"; fi;
		echo "Capacity ($volume_id $volume_used%): SUCCESS" >> $save_path
	fi
done

# --
# End check capacity
# --

# --
# Start S.M.A.R.T. checking
# --

# Fetches just the volume id so we can check S.M.A.R.T. on that volume
my_disks=`diskutil list | grep 'partition_scheme' | awk '{print $5;}'`

for i in $my_disks
do
	if [[ $debug = 1 ]]; then echo "(S.M.A.R.T.) Starting to check on $i"; fi;
	smart_result=`diskutil info $i`
	
	# Must ignore disk images, they always fail S.M.A.R.T and are not worth testing
	is_dmg=`echo "$smart_result" | grep 'Disk Image'`
	if [[ $? -eq 1 ]]; then
		if [[ $debug = 1 ]]; then echo "(S.M.A.R.T.) Found a .dmg, aborting smart check"; fi;
		continue
	fi

	# Parse out the S.M.A.R.T. status
	# grep EXIT STATUS 0 means found, 1 means no match
	smart_result=`echo "$smart_result" | grep -E 'SMART Status\:[[:space:]]+[Verified | Not Supported]'`
	if [[ $? -eq 1 ]]
	then
		if [[ $debug = 1 ]]; then echo "(S.M.A.R.T.) SMART is failing"; fi;
		echo "S.M.A.R.T. status on ($i): FAILURE" >> $save_path
	else
		if [[ $debug = 1 ]]; then echo "(S.M.A.R.T.) SMART passed test on $i"; fi;
		echo "S.M.A.R.T. status on ($i): SUCCESS" >> $save_path
	fi
done

# --
# End check S.M.A.R.T.
# --


# --
# Start RAID checking
# --

# Feteches the entire status of all RAID drives
# Maybe later, iterate them, but I do not have more than one raid to test
raid_result=`diskutil checkRaid`

# As far as I can tell, the data from diskutil will always have
# 'No RAID sets found' if it is not supported

echo "$raid_result" | grep --silent 'No RAID sets found'
if [[ $? -eq 0 ]]
then
	# Do nothing, we are not interested in this test, there is no RAID
	if [[ $debug = 1 ]]; then echo "(RAID) No RAID sets found"; fi;
else
	# Found a RAID set to test, start poking at it
	if [[ $debug = 1 ]]; then echo "(RAID) RAID SETS found"; fi;

	# grep EXIT STATUS 0 means found, 1 means no match
	echo "$raid_result" | grep -i --silent degraded

	# Check the result of the last grep command $? is bash last command
	if [[ $? -eq 0 ]]
	then
		if [[ $debug = 1 ]]; then echo "(RAID) RAID is Degraded"; fi;
		echo "RAID status: FAILURE" >> $save_path
	else
		if [[ $debug = 1 ]]; then echo "(RAID) RAID is Online"; fi;
		echo "RAID status: SUCCESS" >> $save_path
	fi
fi

# --
# End RAID checking
# --

# Show me the results
if [[ $debug = 1 ]]; then cat $save_path; fi;

# grep EXIT STATUS 0 means found, 1 means no match
cat $save_path | grep --silent 'FAILURE'
if [[ $? -eq 0 ]]
then
		if [[ $debug = 1 ]]; then echo "Something is going wrong with your drives"; fi;
		exit 1
else
		if [[ $debug = 1 ]]; then echo "All remote drives passed testing SMART, RAID, and CAPACITY"; fi;
		exit 0
fi

# Change log
# 09:48 PM Wednesday 10/13/2008
# Initial script written by merging three scripts

# 07:31 PM Wednesday 10/15/2008
# Went with "grep -E 'SMART Status\:[[:space:]]+Verified" for the S.M.A.R.T. check
# This should remove any chance for a false positive
# Added in debug variable, and changed the way paths worked for '$save_path'

# 03:43 PM Monday 01/19/2009
# Converted script to work locally, just for Dejal Simon users
# Changed S.M.A.R.T. section grep search string to 'partition_scheme' from
#	apple_partition_scheme since the latter was too strict a string
# Excluded .dmg files from capacity check
# Added in cleaner debug lines, to know where the chcek is at in the run


